"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
const path = __importStar(require("path"));
const constants_1 = require("./constants");
const utils_1 = require("./utils");
const ensureArray = (o) => (Array.isArray(o) ? o : [o]);
const removeIconFiles = (githubApi, githubParams, componentName, componentPath, commitMessagePattern = constants_1.COMMIT_MESSAGE_PATTERNS.DELETE) => __awaiter(void 0, void 0, void 0, function* () {
    const deleteMessage = `remove ${componentName}`;
    const message = utils_1.replaceAll(commitMessagePattern, constants_1.COMMIT_MESSAGE_PLACEHOLDER, deleteMessage);
    const { data: results } = yield githubApi.repos.getContent({
        owner: githubParams.owner,
        path: componentPath,
        ref: githubParams.head,
        repo: githubParams.repo,
    });
    yield utils_1.eagerPromises(ensureArray(results).map(({ path, sha }) => githubApi.repos.deleteFile({
        branch: githubParams.head,
        message,
        owner: githubParams.owner,
        path,
        repo: githubParams.repo,
        sha,
    })));
});
const addOrModifyIconFile = (githubApi, githubParams, componentName, fileName, filePath, fileContents, commitMessagePatternCreate = constants_1.COMMIT_MESSAGE_PATTERNS.CREATE, commitMessagePatternUpdate = constants_1.COMMIT_MESSAGE_PATTERNS.UPDATE, logger = console) => __awaiter(void 0, void 0, void 0, function* () {
    let fileSha;
    let fileContentsOld;
    try {
        const { data } = yield githubApi.repos.getContent({
            owner: githubParams.owner,
            path: filePath,
            ref: githubParams.head,
            repo: githubParams.repo,
        });
        if (Array.isArray(data)) {
            return logger.info('Path is folder, skipping', filePath);
        }
        fileSha = data.sha;
        fileContentsOld = utils_1.fromBase64(data.content);
    }
    catch (e) {
        logger.error(`${e}: ${filePath}`);
    }
    if (fileContentsOld === fileContents) {
        return logger.info('No changes, skipping file', filePath);
    }
    const message = fileSha
        ? utils_1.replaceAll(commitMessagePatternUpdate, constants_1.COMMIT_MESSAGE_PLACEHOLDER, `modify ${componentName} ${path.basename(fileName)}`)
        : utils_1.replaceAll(commitMessagePatternCreate, constants_1.COMMIT_MESSAGE_PLACEHOLDER, `add ${componentName} ${path.basename(fileName)}`);
    yield githubApi.repos.createOrUpdateFileContents({
        branch: githubParams.head,
        content: utils_1.toBase64(fileContents),
        message,
        owner: githubParams.owner,
        path: filePath,
        repo: githubParams.repo,
        sha: fileSha,
    });
});
const writeComponent = (_a) => { var _b, _c, _d, _e, _f, _g, _h, _j, githubApi, githubParams, params; return __awaiter(void 0, void 0, void 0, function* () {
    _j = _a.github, { api: githubApi } = _j, githubParams = __rest(_j, ["api"]), params = __rest(_a, ["github"]);
    if (!params.componentName) {
        return (_b = params.logger) === null || _b === void 0 ? void 0 : _b.warn(`No '${constants_1.STATE.COMPONENT_NAME}' provided for '${params.assetFile}'.`);
    }
    if (!params.outputPath) {
        return (_c = params.logger) === null || _c === void 0 ? void 0 : _c.warn(`No '${constants_1.OPTIONS.OUTPUT_PATH}' provided.`);
    }
    if (!params.fileExt) {
        (_d = params.logger) === null || _d === void 0 ? void 0 : _d.warn(`No '${constants_1.OPTIONS.FILE_EXT}' provided.`);
    }
    // gather files
    const componentFiles = Object.entries(params.componentFiles);
    const singleFile = componentFiles.length === 0;
    const pathToFolder = path.join(params.folderPath, params.outputPath, singleFile ? '' : params.componentName);
    const componentFileName = (singleFile ? params.componentName : 'index') +
        (params.fileExt ? `.${params.fileExt}` : '');
    componentFiles.push([componentFileName, params.code]);
    const componentFilePath = path.join(pathToFolder, componentFileName);
    // commit file changes
    const pendingPromises = [];
    if (params.diffType === constants_1.STATUSES.REMOVED) {
        pendingPromises.push(removeIconFiles(githubApi, githubParams, params.componentName, singleFile ? componentFilePath : pathToFolder, (_e = params.commitMessagePatterns) === null || _e === void 0 ? void 0 : _e.delete));
    }
    else {
        // added, modified or renamed
        for (const [fileName, fileContents] of componentFiles) {
            const filePath = path.normalize(path.join(pathToFolder, fileName));
            pendingPromises.push(addOrModifyIconFile(githubApi, githubParams, params.componentName, fileName, filePath, fileContents, (_f = params.commitMessagePatterns) === null || _f === void 0 ? void 0 : _f.create, (_g = params.commitMessagePatterns) === null || _g === void 0 ? void 0 : _g.update, params.logger));
        }
        if (params.diffType === constants_1.STATUSES.RENAMED && params.componentNameOld) {
            const oldPathToFolder = path.join(params.folderPath, params.outputPath, singleFile ? '' : params.componentNameOld);
            const oldPathToFile = path.join(oldPathToFolder, singleFile ? params.componentNameOld : 'index');
            const oldComponentFilePath = params.fileExt
                ? `${oldPathToFile}.${params.fileExt}`
                : oldPathToFile;
            pendingPromises.push(removeIconFiles(githubApi, githubParams, params.componentNameOld, singleFile ? oldComponentFilePath : oldPathToFolder, (_h = params.commitMessagePatterns) === null || _h === void 0 ? void 0 : _h.delete));
        }
    }
    yield utils_1.eagerPromises(pendingPromises);
}); };
exports.default = writeComponent;
